window.Gleam.initSearch([{"doc":"gpkm","title":"gpkm","content":"# gpkm\n\nA Gleam PKM reader\n\n[![Package Version](https://img.shields.io/hexpm/v/gpkm)](https://hex.pm/packages/gpkm)\n[![Hex Docs](https://img.shields.io/badge/hex-docs-ffaff3)](https://hexdocs.pm/gpkm/)\n\n```sh\ngleam add gpkm\n```\n\n## What is it ?\n\nA simple deserializer, to read PKM data (Gen4, Gen5):\n\n- `.pkm`, `.pk4`, `.pk5`, `.sav` (experimental) files\n- `base64` pkm strings (.pkm bytes as base64)\n- `base64` pkm files\n\nThe read data is used to craft a `Pkm` record,\ncontaining human readable data:\n\n```gleam\n  Pkm(\n    pid: Option(Int),\n    nickname: Option(String),\n    national_pokedex_id: Option(Int),\n    held_item: Option(String),\n    ot_name: Option(String),\n    ot_id: Option(Int),\n    ot_secret_id: Option(Int),\n    moves: Option(Moves),\n    ability: Option(String),\n    individual_values: Option(IndividualValues),\n    effort_values: Option(EffortValues),\n    experience_points: Option(Int),\n    friendship: Option(Int),\n    original_language: Option(String),\n    shiny: Option(Bool),\n    level: Option(Int),\n    nature: Option(String),\n    species: Option(String),\n    gender: Option(String),\n    hidden_power: Option(HiddenPower),\n    b64_pkm_data: String,\n  )\n```\n\n## Motivation\n\nActually, mostly for fun\n\nI&#39;ve made a few ones in other languages (Scala, Python, Zig...),\nbut I wanted to make one in Gleam\n\nBut, I&#39;m also gonna make a little web service to display info, from a PKM file,\nthis tool will be used to convert PKM data to JSON\n\nIt will evolve later:\n\n- better documentation\n- more test coverage\n- better decrypt logic\n- add encrypt logic\n\n## Usage\n\n```gleam\nimport gleam/io\nimport gpkm/pkm/pkm_bytes.{B64, Bin, PkmBits, PkmFile}\nimport gpkm/pkm/ser_de.{read_pkm}\n\npub fn main() {\n  let chimchar_fr_pkm_path = &quot;misc/chimchar_fr.pkm&quot;\n\n  let chimchar_fr_pkm_b64 = &quot;\n    PbLkNgAAN1KGAQAAB8k5xpcAAABGQgADAAAAAQAAAAAAAAAAAAAAAAoAKwAAAAAAIx4AAAAAAABh\n    4Dc+AAAAAAAAAAAAAAAAOQE/ATMBPQE+ATMBLQE8ASsBNwH//wAKAAAAAAAAAABWAUkBSAFRAVMB\n    UwFSAf//AAAACwEDAABMAAAEBQwAAAAAAAAFABMAEwALAAkACwAKAAoAAAAAAAADCv//////////\n    ////////////////////AAD//wAA////////AAD///////9qAf////8AAAAAAAAAAAAAAAAAAAAA\n    AAAAAAAAAAA=\n  &quot;\n\n  let _pkm = read_pkm(chimchar_fr_pkm_path, PkmFile(Bin)) |&gt; io.debug\n\n  let _pkm = read_pkm(chimchar_fr_pkm_b64, PkmBits(B64)) |&gt; io.debug\n}\n```\n\nMore examples in the [ser_de_test.gleam](./test/pkm/ser_de_test.gleam) file\n\nFurther documentation can be found at &lt;https://hexdocs.pm/gpkm&gt;.\n\n## Development\n\n```sh\ngleam run   # Run the project\ngleam test  # Run the tests\n```\n","url":"index.html"},{"doc":"gpkm/pkm/lookup_tables/game_chars","title":"ingame_chars","content":"pub const ingame_chars: List(Int)\n\nSynonyms:\ngpkm/pkm/lookup_tables/game_chars.ingame_chars\ngpkm/pkm/lookup_tables/game_chars ingame_chars","url":"gpkm/pkm/lookup_tables/game_chars.html#ingame_chars"},{"doc":"gpkm/pkm/lookup_tables/game_chars","title":"unicode_chars","content":"pub const unicode_chars: List(Int)\n\nSynonyms:\ngpkm/pkm/lookup_tables/game_chars.unicode_chars\ngpkm/pkm/lookup_tables/game_chars unicode_chars","url":"gpkm/pkm/lookup_tables/game_chars.html#unicode_chars"},{"doc":"gpkm/pkm/lookup_tables/game_chars","title":"gpkm/pkm/lookup_tables/game_chars","content":"","url":"gpkm/pkm/lookup_tables/game_chars.html"},{"doc":"gpkm/utils/bytes","title":"Bytes","content":"pub type Bytes =\n  List(Int)\n\n\nSynonyms:\ngpkm/utils/bytes.Bytes\ngpkm/utils/bytes Bytes","url":"gpkm/utils/bytes.html#Bytes"},{"doc":"gpkm/utils/bytes","title":"Chunks","content":"pub type Chunks =\n  List(Bytes)\n\n\nSynonyms:\ngpkm/utils/bytes.Chunks\ngpkm/utils/bytes Chunks","url":"gpkm/utils/bytes.html#Chunks"},{"doc":"gpkm/utils/bytes","title":"Endian","content":"pub type Endian {\n  BigEndian\n  LittleEndian\n}\n\nBigEndian\n\n\nLittleEndian\n\n\nSynonyms:\ngpkm/utils/bytes.Endian\ngpkm/utils/bytes Endian","url":"gpkm/utils/bytes.html#Endian"},{"doc":"gpkm/utils/bytes","title":"at","content":"pub fn at(xs: List(Int), index: Int) -&gt; List(Int)\n Makes a single-element slice from a bytes list\n at a given index\n\n If the index is out of bounds, an empty bytes list is returned\n \n \n [0,1,2,3,4,5] |&gt; at(2)\n // -&gt; [2]\n\n [0,1,2,3,4,5] |&gt; at(42)\n // -&gt; []\n\n [] |&gt; at(42)\n // -&gt; []\n \n\nSynonyms:\ngpkm/utils/bytes.at\ngpkm/utils/bytes at","url":"gpkm/utils/bytes.html#at"},{"doc":"gpkm/utils/bytes","title":"bits_to_bytes","content":"pub fn bits_to_bytes(bits: BitArray) -&gt; List(Int)\n\nSynonyms:\ngpkm/utils/bytes.bits_to_bytes\ngpkm/utils/bytes bits_to_bytes","url":"gpkm/utils/bytes.html#bits_to_bytes"},{"doc":"gpkm/utils/bytes","title":"bytes_to_bits","content":"pub fn bytes_to_bits(bytes: List(Int)) -&gt; BitArray\n\nSynonyms:\ngpkm/utils/bytes.bytes_to_bits\ngpkm/utils/bytes bytes_to_bits","url":"gpkm/utils/bytes.html#bytes_to_bits"},{"doc":"gpkm/utils/bytes","title":"chunked_slice","content":"pub fn chunked_slice(\n  xs: List(Int),\n  start: Int,\n  end: Int,\n  chunk_size: Int,\n  endian: Endian,\n) -&gt; List(List(Int))\n Makes sized chunks from a slice of a bytes list,\n reordering each chunk according to the target endian\n\n \n [0,1,2,3,4,5,6,7,8,9] |&gt; chunked_slice(1, 8, 2, BigEndian)\n // -&gt; [[1,2], [3,4], [5,6], [7,8]]\n\n [0,1,2,3,4,5,6,7,8,9] |&gt; chunked_slice(1, 8, 4, LittleEndian)\n // -&gt; [[4,3,2,1], [8,7,6,5]]\n \n\nSynonyms:\ngpkm/utils/bytes.chunked_slice\ngpkm/utils/bytes chunked_slice","url":"gpkm/utils/bytes.html#chunked_slice"},{"doc":"gpkm/utils/bytes","title":"i16_to_i8_bytes","content":"pub fn i16_to_i8_bytes(i16: Int, endian: Endian) -&gt; List(Int)\n Converts a 16 bits int, to an 8 bits int, as a bytes list\n\n \n 0x0123 |&gt; i16_to_i8_bytes(BigEndian)\n // -&gt; [0x01, 0x23]\n\n 0x0123 |&gt; i16_to_i8_bytes(LittleEndian)\n // -&gt; [0x23, 0x01]\n \n\nSynonyms:\ngpkm/utils/bytes.i16_to_i8_bytes\ngpkm/utils/bytes i16_to_i8_bytes","url":"gpkm/utils/bytes.html#i16_to_i8_bytes"},{"doc":"gpkm/utils/bytes","title":"is_bit_set","content":"pub fn is_bit_set(number x: Int, bit_position n: Int) -&gt; Bool\n Checks if nth bit (from right to left) of a number,\n is set (!= 0) using bitwise\n \n let num = 10 // 10 == 0 b 1 0 1 0  // number (as bits)\n              //           ^ ^ ^ ^\n              //           3 2 1 0  // bit_position\n\n is_bit_set(num, 3)\n // -&gt; True\n \n is_bit_set(num, 2)\n // -&gt; False\n \n is_bit_set(num, 1)\n // -&gt; True\n \n\nSynonyms:\ngpkm/utils/bytes.is_bit_set\ngpkm/utils/bytes is_bit_set","url":"gpkm/utils/bytes.html#is_bit_set"},{"doc":"gpkm/utils/bytes","title":"ordered_slice","content":"pub fn ordered_slice(\n  xs: List(Int),\n  start: Int,\n  end: Int,\n  endian: Endian,\n) -&gt; List(Int)\n Extracts a slice from a bytes list\n reordering it according to a target endian\n\n \n [0,1,2,3,4,5] |&gt; ordered_slice(1, 4, BigEndian)\n // -&gt; [1,2,3,4]\n\n [0,1,2,3,4,5] |&gt; ordered_slice(1, 4, LittleEndian)\n // -&gt; [4,3,2,1]\n \n\nSynonyms:\ngpkm/utils/bytes.ordered_slice\ngpkm/utils/bytes ordered_slice","url":"gpkm/utils/bytes.html#ordered_slice"},{"doc":"gpkm/utils/bytes","title":"slice","content":"pub fn slice(bs: List(Int), start: Int, end: Int) -&gt; List(Int)\n Extracts a slice from a bytes list\n\n \n [0,1,2,3,4,5] |&gt; slice(1, 4)\n // -&gt; [1,2,3,4]\n\n [0,1,2,3,4,5] |&gt; slice(2, 2)\n // -&gt; [2]\n \n\nSynonyms:\ngpkm/utils/bytes.slice\ngpkm/utils/bytes slice","url":"gpkm/utils/bytes.html#slice"},{"doc":"gpkm/utils/bytes","title":"take_upper_16_bits","content":"pub fn take_upper_16_bits(number: Int) -&gt; Int\n Takes the 16 leftmost bits from the 32 rightmost bits of a numbe\n\n To return the &quot;upper 16 bits&quot; of the rand number (that may be bigger than 32bits)\n the number should first be cast as a 32bits integer (using a 32bits bitmask)\n then we can take its first 16 bits (by shifting 16bits to the right)\n\n \n take_upper_16_bits(0xdeadbeef)\n // -&gt; 0xdead\n\n take_upper_16_bits(0xf00d1337deadbeef)\n // -&gt; 0xdead\n\n take_upper_16_bits(0b00001111000011110000000000000000)\n // -&gt; 0b0000111100001111\n \n\n A very uneffective implementation could also be:\n\n \n number\n |&gt; int.to_base2\n |&gt; string.slice(-32, 16)\n |&gt; int.base_parse(2)\n |&gt; result.unwrap(0)\n \n\nSynonyms:\ngpkm/utils/bytes.take_upper_16_bits\ngpkm/utils/bytes take_upper_16_bits","url":"gpkm/utils/bytes.html#take_upper_16_bits"},{"doc":"gpkm/utils/bytes","title":"to_int","content":"pub fn to_int(xs: List(Int)) -&gt; Int\n Converts a bytes list to a number, by concatenating the bytes\n\n \n // example when: max_byte_value = 0xff\n\n [0x42, 0xff] |&gt; int.undigits(0xff + 1)\n // -&gt; 0x42ff\n\n [0xde, 0xad, 0xbe, 0xef] |&gt; int.undigits(0xff + 1)\n // -&gt; 0xdeadbeef\n \n\nSynonyms:\ngpkm/utils/bytes.to_int\ngpkm/utils/bytes to_int","url":"gpkm/utils/bytes.html#to_int"},{"doc":"gpkm/utils/bytes","title":"gpkm/utils/bytes","content":"","url":"gpkm/utils/bytes.html"},{"doc":"gpkm/pkm/lookup_tables/lookup","title":"lookup_ingame_str","content":"pub fn lookup_ingame_str(\n  key: List(Int),\n  lookup table: List(#(Int, String)),\n) -&gt; Result(String, Nil)\n Get a lookup table (mainly from pkm_lookup_tables)\n converts it to a dictionary,\n and get the string corresponding to the in-game bytes key\n\nSynonyms:\ngpkm/pkm/lookup_tables/lookup.lookup_ingame_str\ngpkm/pkm/lookup_tables/lookup lookup_ingame_str","url":"gpkm/pkm/lookup_tables/lookup.html#lookup_ingame_str"},{"doc":"gpkm/pkm/lookup_tables/lookup","title":"gpkm/pkm/lookup_tables/lookup","content":"","url":"gpkm/pkm/lookup_tables/lookup.html"},{"doc":"gpkm/pkm/lookup_tables/pokemon_info","title":"abilities","content":"pub const abilities: List(#(Int, String))\n\nSynonyms:\ngpkm/pkm/lookup_tables/pokemon_info.abilities\ngpkm/pkm/lookup_tables/pokemon_info abilities","url":"gpkm/pkm/lookup_tables/pokemon_info.html#abilities"},{"doc":"gpkm/pkm/lookup_tables/pokemon_info","title":"held_items","content":"pub const held_items: List(#(Int, String))\n\nSynonyms:\ngpkm/pkm/lookup_tables/pokemon_info.held_items\ngpkm/pkm/lookup_tables/pokemon_info held_items","url":"gpkm/pkm/lookup_tables/pokemon_info.html#held_items"},{"doc":"gpkm/pkm/lookup_tables/pokemon_info","title":"hidden_power_types","content":"pub const hidden_power_types: List(#(Int, String))\n\nSynonyms:\ngpkm/pkm/lookup_tables/pokemon_info.hidden_power_types\ngpkm/pkm/lookup_tables/pokemon_info hidden_power_types","url":"gpkm/pkm/lookup_tables/pokemon_info.html#hidden_power_types"},{"doc":"gpkm/pkm/lookup_tables/pokemon_info","title":"moves","content":"pub const moves: List(#(Int, String))\n\nSynonyms:\ngpkm/pkm/lookup_tables/pokemon_info.moves\ngpkm/pkm/lookup_tables/pokemon_info moves","url":"gpkm/pkm/lookup_tables/pokemon_info.html#moves"},{"doc":"gpkm/pkm/lookup_tables/pokemon_info","title":"natures","content":"pub const natures: List(#(Int, String))\n\nSynonyms:\ngpkm/pkm/lookup_tables/pokemon_info.natures\ngpkm/pkm/lookup_tables/pokemon_info natures","url":"gpkm/pkm/lookup_tables/pokemon_info.html#natures"},{"doc":"gpkm/pkm/lookup_tables/pokemon_info","title":"species","content":"pub const species: List(#(Int, String))\n\nSynonyms:\ngpkm/pkm/lookup_tables/pokemon_info.species\ngpkm/pkm/lookup_tables/pokemon_info species","url":"gpkm/pkm/lookup_tables/pokemon_info.html#species"},{"doc":"gpkm/pkm/lookup_tables/pokemon_info","title":"gpkm/pkm/lookup_tables/pokemon_info","content":"","url":"gpkm/pkm/lookup_tables/pokemon_info.html"},{"doc":"gpkm/pkm/pkm_bytes","title":"EffortValues","content":"pub type EffortValues {\n  EffortValues(\n    hp: Int,\n    atk: Int,\n    def: Int,\n    spe: Int,\n    spa: Int,\n    spd: Int,\n  )\n}\n\nEffortValues(\n  hp: Int,\n  atk: Int,\n  def: Int,\n  spe: Int,\n  spa: Int,\n  spd: Int,\n)\n\n\nSynonyms:\ngpkm/pkm/pkm_bytes.EffortValues\ngpkm/pkm/pkm_bytes EffortValues","url":"gpkm/pkm/pkm_bytes.html#EffortValues"},{"doc":"gpkm/pkm/pkm_bytes","title":"Encoding","content":"pub type Encoding {\n  Bin\n  B64\n}\n\nBin\n\n\nB64\n\n\nSynonyms:\ngpkm/pkm/pkm_bytes.Encoding\ngpkm/pkm/pkm_bytes Encoding","url":"gpkm/pkm/pkm_bytes.html#Encoding"},{"doc":"gpkm/pkm/pkm_bytes","title":"EncryptedDataType","content":"pub type EncryptedDataType {\n  PkmBits(Encoding)\n  SavBits(Encoding)\n  SavPkmBits(Encoding)\n  PkmFile(Encoding)\n  SavFile(Encoding)\n  SavPkmFile(Encoding)\n}\n\nPkmBits(Encoding)\n\n\nSavBits(Encoding)\n\n\nSavPkmBits(Encoding)\n\n\nPkmFile(Encoding)\n\n\nSavFile(Encoding)\n\n\nSavPkmFile(Encoding)\n\n\nSynonyms:\ngpkm/pkm/pkm_bytes.EncryptedDataType\ngpkm/pkm/pkm_bytes EncryptedDataType","url":"gpkm/pkm/pkm_bytes.html#EncryptedDataType"},{"doc":"gpkm/pkm/pkm_bytes","title":"HiddenPower","content":"pub type HiddenPower {\n  HiddenPower(power_type: String, base_power: Int)\n}\n\nHiddenPower(power_type: String, base_power: Int)\n\n\nSynonyms:\ngpkm/pkm/pkm_bytes.HiddenPower\ngpkm/pkm/pkm_bytes HiddenPower","url":"gpkm/pkm/pkm_bytes.html#HiddenPower"},{"doc":"gpkm/pkm/pkm_bytes","title":"IndividualValues","content":"pub type IndividualValues {\n  IndividualValues(\n    hp: Int,\n    atk: Int,\n    def: Int,\n    spe: Int,\n    spa: Int,\n    spd: Int,\n  )\n}\n\nIndividualValues(\n  hp: Int,\n  atk: Int,\n  def: Int,\n  spe: Int,\n  spa: Int,\n  spd: Int,\n)\n\n\nSynonyms:\ngpkm/pkm/pkm_bytes.IndividualValues\ngpkm/pkm/pkm_bytes IndividualValues","url":"gpkm/pkm/pkm_bytes.html#IndividualValues"},{"doc":"gpkm/pkm/pkm_bytes","title":"Move","content":"pub type Move {\n  Move(name: String, pp: Int)\n}\n\nMove(name: String, pp: Int)\n\n\nSynonyms:\ngpkm/pkm/pkm_bytes.Move\ngpkm/pkm/pkm_bytes Move","url":"gpkm/pkm/pkm_bytes.html#Move"},{"doc":"gpkm/pkm/pkm_bytes","title":"Moves","content":"pub type Moves {\n  Moves(\n    move_1: Result(Move, Nil),\n    move_2: Result(Move, Nil),\n    move_3: Result(Move, Nil),\n    move_4: Result(Move, Nil),\n  )\n}\n\nMoves(\n  move_1: Result(Move, Nil),\n  move_2: Result(Move, Nil),\n  move_3: Result(Move, Nil),\n  move_4: Result(Move, Nil),\n)\n\n\nSynonyms:\ngpkm/pkm/pkm_bytes.Moves\ngpkm/pkm/pkm_bytes Moves","url":"gpkm/pkm/pkm_bytes.html#Moves"},{"doc":"gpkm/pkm/pkm_bytes","title":"UnencryptedHeaderBytes","content":"pub type UnencryptedHeaderBytes {\n  UnencryptedHeaderBytes(pid: Bytes, tmp: Bytes, checksum: Bytes)\n}\n\nUnencryptedHeaderBytes(pid: Bytes, tmp: Bytes, checksum: Bytes)\n\n\nSynonyms:\ngpkm/pkm/pkm_bytes.UnencryptedHeaderBytes\ngpkm/pkm/pkm_bytes UnencryptedHeaderBytes","url":"gpkm/pkm/pkm_bytes.html#UnencryptedHeaderBytes"},{"doc":"gpkm/pkm/pkm_bytes","title":"UnencryptedPkmBytes","content":"pub type UnencryptedPkmBytes {\n  UnencryptedPkmBytes(\n    pid: Bytes,\n    tmp: Bytes,\n    checksum: Bytes,\n    national_pokedex_id: Bytes,\n    held_item: Bytes,\n    ot_id: Bytes,\n    ot_secret_id: Bytes,\n    experience_points: Bytes,\n    friendship: Bytes,\n    ability: Bytes,\n    markings: Bytes,\n    original_language: Bytes,\n    hp_effort_value: Bytes,\n    attack_effort_value: Bytes,\n    defense_effort_value: Bytes,\n    speed_effort_value: Bytes,\n    sp_attack_effort_value: Bytes,\n    sp_defense_effort_value: Bytes,\n    cool_contest_value: Bytes,\n    beauty_contest_value: Bytes,\n    cute_contest_value: Bytes,\n    smart_contest_value: Bytes,\n    tough_contest_value: Bytes,\n    sheen_contest_value: Bytes,\n    sinnoh_ribbon_set_1: Bytes,\n    sinnoh_ribbon_set_2: Bytes,\n    move_1_id: Bytes,\n    move_2_id: Bytes,\n    move_3_id: Bytes,\n    move_4_id: Bytes,\n    move_1_current_pp: Bytes,\n    move_2_current_pp: Bytes,\n    move_3_current_pp: Bytes,\n    move_4_current_pp: Bytes,\n    move_pp_ups: Bytes,\n    individual_values: Bytes,\n    hoenn_ribbon_set_1: Bytes,\n    hoenn_ribbon_set_2: Bytes,\n    encounter_main_info: Bytes,\n    hgss_shiny_leaves: Bytes,\n    unused_b_1: Bytes,\n    egg_platinum_location: Bytes,\n    met_at_platinum_location: Bytes,\n    nickname: Bytes,\n    unused_c_1: Bytes,\n    origin_game: Bytes,\n    sinnoh_ribbon_set_3: Bytes,\n    sinnoh_ribbon_set_4: Bytes,\n    unused_c_2: Bytes,\n    ot_name: Bytes,\n    date_egg_received: Bytes,\n    date_met: Bytes,\n    egg_diamond_pearl_location: Bytes,\n    met_at_diamond_pearl_location: Bytes,\n    pokerus: Bytes,\n    poke_ball: Bytes,\n    encounter_bonus_info: Bytes,\n    encounter_type: Bytes,\n    hgss_poke_ball: Bytes,\n    unused_d_1: Bytes,\n    current_status: Bytes,\n    unknown_1: Bytes,\n    unknown_2: Bytes,\n    level: Bytes,\n    seals_capsule_index: Bytes,\n    current_hp: Bytes,\n    max_hp: Bytes,\n    attack: Bytes,\n    defense: Bytes,\n    speed: Bytes,\n    special_attack: Bytes,\n    special_defense: Bytes,\n    unknown_3: Bytes,\n    seal_coordinates: Bytes,\n    b64_pkm: String,\n  )\n}\n\nUnencryptedPkmBytes(\n  pid: Bytes,\n  tmp: Bytes,\n  checksum: Bytes,\n  national_pokedex_id: Bytes,\n  held_item: Bytes,\n  ot_id: Bytes,\n  ot_secret_id: Bytes,\n  experience_points: Bytes,\n  friendship: Bytes,\n  ability: Bytes,\n  markings: Bytes,\n  original_language: Bytes,\n  hp_effort_value: Bytes,\n  attack_effort_value: Bytes,\n  defense_effort_value: Bytes,\n  speed_effort_value: Bytes,\n  sp_attack_effort_value: Bytes,\n  sp_defense_effort_value: Bytes,\n  cool_contest_value: Bytes,\n  beauty_contest_value: Bytes,\n  cute_contest_value: Bytes,\n  smart_contest_value: Bytes,\n  tough_contest_value: Bytes,\n  sheen_contest_value: Bytes,\n  sinnoh_ribbon_set_1: Bytes,\n  sinnoh_ribbon_set_2: Bytes,\n  move_1_id: Bytes,\n  move_2_id: Bytes,\n  move_3_id: Bytes,\n  move_4_id: Bytes,\n  move_1_current_pp: Bytes,\n  move_2_current_pp: Bytes,\n  move_3_current_pp: Bytes,\n  move_4_current_pp: Bytes,\n  move_pp_ups: Bytes,\n  individual_values: Bytes,\n  hoenn_ribbon_set_1: Bytes,\n  hoenn_ribbon_set_2: Bytes,\n  encounter_main_info: Bytes,\n  hgss_shiny_leaves: Bytes,\n  unused_b_1: Bytes,\n  egg_platinum_location: Bytes,\n  met_at_platinum_location: Bytes,\n  nickname: Bytes,\n  unused_c_1: Bytes,\n  origin_game: Bytes,\n  sinnoh_ribbon_set_3: Bytes,\n  sinnoh_ribbon_set_4: Bytes,\n  unused_c_2: Bytes,\n  ot_name: Bytes,\n  date_egg_received: Bytes,\n  date_met: Bytes,\n  egg_diamond_pearl_location: Bytes,\n  met_at_diamond_pearl_location: Bytes,\n  pokerus: Bytes,\n  poke_ball: Bytes,\n  encounter_bonus_info: Bytes,\n  encounter_type: Bytes,\n  hgss_poke_ball: Bytes,\n  unused_d_1: Bytes,\n  current_status: Bytes,\n  unknown_1: Bytes,\n  unknown_2: Bytes,\n  level: Bytes,\n  seals_capsule_index: Bytes,\n  current_hp: Bytes,\n  max_hp: Bytes,\n  attack: Bytes,\n  defense: Bytes,\n  speed: Bytes,\n  special_attack: Bytes,\n  special_defense: Bytes,\n  unknown_3: Bytes,\n  seal_coordinates: Bytes,\n  b64_pkm: String,\n)\n\n\nSynonyms:\ngpkm/pkm/pkm_bytes.UnencryptedPkmBytes\ngpkm/pkm/pkm_bytes UnencryptedPkmBytes","url":"gpkm/pkm/pkm_bytes.html#UnencryptedPkmBytes"},{"doc":"gpkm/pkm/pkm_bytes","title":"get_evs","content":"pub fn get_evs(pkm_bytes: UnencryptedPkmBytes) -&gt; EffortValues\n Each EV is a single-byte value\n\nSynonyms:\ngpkm/pkm/pkm_bytes.get_evs\ngpkm/pkm/pkm_bytes get_evs","url":"gpkm/pkm/pkm_bytes.html#get_evs"},{"doc":"gpkm/pkm/pkm_bytes","title":"get_ivs","content":"pub fn get_ivs(data: List(Int)) -&gt; IndividualValues\n Extract each IV from a 32bits (4 bytes) Int\n More info in the UnencryptedPkmBytes type definition\n\n The 32bits Int also hold 2 flags (isEgg, isNicknamed),\n but I don&#39;t use them for now (maybe later, to make a more exhaustive JSON)\n\nSynonyms:\ngpkm/pkm/pkm_bytes.get_ivs\ngpkm/pkm/pkm_bytes get_ivs","url":"gpkm/pkm/pkm_bytes.html#get_ivs"},{"doc":"gpkm/pkm/pkm_bytes","title":"get_moves","content":"pub fn get_moves(upbs: UnencryptedPkmBytes) -&gt; Moves\n Get moves and their pps\n Each move/pp combo will be a Result(Move, Nil),\n as a Pokemon might not always have learned 4 moves\n\nSynonyms:\ngpkm/pkm/pkm_bytes.get_moves\ngpkm/pkm/pkm_bytes get_moves","url":"gpkm/pkm/pkm_bytes.html#get_moves"},{"doc":"gpkm/pkm/pkm_bytes","title":"get_name","content":"pub fn get_name(data: List(Int)) -&gt; String\n As the in-game names are composed by non-standard unicode chars\n they need to be deserialized using a lookup table\n\n In-game names (or nicknames), are little endian bytes lists with variable lengths.\n\n Each encoded letter is a little endian 16bits (2 bytes) value,\n used as an index of the corresponding unicode letter, from the lookup table\n\n When the name&#39;s length is smaller than the its reserved offset length,\n an EOL byte (0xffff) is used to determine the end of a name.\n\n \n Example:\n - field:   Origin Trainer Name\n - offset:  0x68-0x77 (length: 15)\n - OT name: &quot;John&quot; (length: 4)\n - bytes:   [ 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0xff, 0xff, &#39;n&#39;, &#39;h&#39;, &#39;o&#39;, &#39;J&#39; ]\n                                                         ^    ^    ^    ^\n                          each letter here is used to visualize the logic but\n                          is actually a 2-bytes value (the index of the letter)\n \n\nSynonyms:\ngpkm/pkm/pkm_bytes.get_name\ngpkm/pkm/pkm_bytes get_name","url":"gpkm/pkm/pkm_bytes.html#get_name"},{"doc":"gpkm/pkm/pkm_bytes","title":"get_shifted_iv","content":"pub fn get_shifted_iv(ivs: Int, by: Int) -&gt; Int\n Use IV max value: `0x1f` (a.k.a `31`), as bit mask to extract target IV\n\n c\n IVS &gt;&gt; BY &amp; MASK\n \n\nSynonyms:\ngpkm/pkm/pkm_bytes.get_shifted_iv\ngpkm/pkm/pkm_bytes get_shifted_iv","url":"gpkm/pkm/pkm_bytes.html#get_shifted_iv"},{"doc":"gpkm/pkm/pkm_bytes","title":"lookup_move","content":"pub fn lookup_move(\n  move_id: List(Int),\n  move_pp: List(Int),\n) -&gt; Result(Move, Nil)\n Get move names from the &#39;moves&#39; lookup table\n\nSynonyms:\ngpkm/pkm/pkm_bytes.lookup_move\ngpkm/pkm/pkm_bytes lookup_move","url":"gpkm/pkm/pkm_bytes.html#lookup_move"},{"doc":"gpkm/pkm/pkm_bytes","title":"gpkm/pkm/pkm_bytes","content":"","url":"gpkm/pkm/pkm_bytes.html"},{"doc":"gpkm/pkm/lookup_tables/game_info","title":"game","content":"pub const game: List(#(Int, String))\n\nSynonyms:\ngpkm/pkm/lookup_tables/game_info.game\ngpkm/pkm/lookup_tables/game_info game","url":"gpkm/pkm/lookup_tables/game_info.html#game"},{"doc":"gpkm/pkm/lookup_tables/game_info","title":"language","content":"pub const language: List(#(Int, String))\n\nSynonyms:\ngpkm/pkm/lookup_tables/game_info.language\ngpkm/pkm/lookup_tables/game_info language","url":"gpkm/pkm/lookup_tables/game_info.html#language"},{"doc":"gpkm/pkm/lookup_tables/game_info","title":"gpkm/pkm/lookup_tables/game_info","content":"","url":"gpkm/pkm/lookup_tables/game_info.html"},{"doc":"gpkm/pkm/pkm_builder","title":"Pkm","content":"pub type Pkm {\n  Pkm(\n    pid: Option(Int),\n    nickname: Option(String),\n    national_pokedex_id: Option(Int),\n    held_item: Option(String),\n    ot_name: Option(String),\n    ot_id: Option(Int),\n    ot_secret_id: Option(Int),\n    moves: Option(pkm_bytes.Moves),\n    ability: Option(String),\n    individual_values: Option(pkm_bytes.IndividualValues),\n    effort_values: Option(pkm_bytes.EffortValues),\n    experience_points: Option(Int),\n    friendship: Option(Int),\n    original_language: Option(String),\n    shiny: Option(Bool),\n    level: Option(Int),\n    nature: Option(String),\n    species: Option(String),\n    gender: Option(String),\n    hidden_power: Option(pkm_bytes.HiddenPower),\n    b64_pkm_data: String,\n  )\n}\n\nPkm(\n  pid: Option(Int),\n  nickname: Option(String),\n  national_pokedex_id: Option(Int),\n  held_item: Option(String),\n  ot_name: Option(String),\n  ot_id: Option(Int),\n  ot_secret_id: Option(Int),\n  moves: Option(pkm_bytes.Moves),\n  ability: Option(String),\n  individual_values: Option(pkm_bytes.IndividualValues),\n  effort_values: Option(pkm_bytes.EffortValues),\n  experience_points: Option(Int),\n  friendship: Option(Int),\n  original_language: Option(String),\n  shiny: Option(Bool),\n  level: Option(Int),\n  nature: Option(String),\n  species: Option(String),\n  gender: Option(String),\n  hidden_power: Option(pkm_bytes.HiddenPower),\n  b64_pkm_data: String,\n)\n\n\nSynonyms:\ngpkm/pkm/pkm_builder.Pkm\ngpkm/pkm/pkm_builder Pkm","url":"gpkm/pkm/pkm_builder.html#Pkm"},{"doc":"gpkm/pkm/pkm_builder","title":"get_nature","content":"pub fn get_nature(pid: Int) -&gt; Result(String, Nil)\n\nSynonyms:\ngpkm/pkm/pkm_builder.get_nature\ngpkm/pkm/pkm_builder get_nature","url":"gpkm/pkm/pkm_builder.html#get_nature"},{"doc":"gpkm/pkm/pkm_builder","title":"get_species","content":"pub fn get_species(\n  national_pokedex_id: Int,\n) -&gt; Result(String, Nil)\n\nSynonyms:\ngpkm/pkm/pkm_builder.get_species\ngpkm/pkm/pkm_builder get_species","url":"gpkm/pkm/pkm_builder.html#get_species"},{"doc":"gpkm/pkm/pkm_builder","title":"is_shiny","content":"pub fn is_shiny(pid: Int, ot_id: Int, secret_id: Int) -&gt; Bool\n\nSynonyms:\ngpkm/pkm/pkm_builder.is_shiny\ngpkm/pkm/pkm_builder is_shiny","url":"gpkm/pkm/pkm_builder.html#is_shiny"},{"doc":"gpkm/pkm/pkm_builder","title":"new","content":"pub fn new(b64_pkm_data: String) -&gt; Pkm\n\nSynonyms:\ngpkm/pkm/pkm_builder.new\ngpkm/pkm/pkm_builder new","url":"gpkm/pkm/pkm_builder.html#new"},{"doc":"gpkm/pkm/pkm_builder","title":"with_ability","content":"pub fn with_ability(pkm: Pkm, ability: List(Int)) -&gt; Pkm\n\nSynonyms:\ngpkm/pkm/pkm_builder.with_ability\ngpkm/pkm/pkm_builder with_ability","url":"gpkm/pkm/pkm_builder.html#with_ability"},{"doc":"gpkm/pkm/pkm_builder","title":"with_effort_values","content":"pub fn with_effort_values(\n  pkm: Pkm,\n  bs: UnencryptedPkmBytes,\n) -&gt; Pkm\n Uses the whole UnencryptedPkmBytes to build Pkm with effort values\n\n As each effort_value is a field of UnencryptedPkmBytes\n each effort_value will be extracted from the `get_evs` function\n\nSynonyms:\ngpkm/pkm/pkm_builder.with_effort_values\ngpkm/pkm/pkm_builder with_effort_values","url":"gpkm/pkm/pkm_builder.html#with_effort_values"},{"doc":"gpkm/pkm/pkm_builder","title":"with_experience_points","content":"pub fn with_experience_points(\n  pkm: Pkm,\n  experience_points: List(Int),\n) -&gt; Pkm\n\nSynonyms:\ngpkm/pkm/pkm_builder.with_experience_points\ngpkm/pkm/pkm_builder with_experience_points","url":"gpkm/pkm/pkm_builder.html#with_experience_points"},{"doc":"gpkm/pkm/pkm_builder","title":"with_friendship","content":"pub fn with_friendship(pkm: Pkm, friendship: List(Int)) -&gt; Pkm\n\nSynonyms:\ngpkm/pkm/pkm_builder.with_friendship\ngpkm/pkm/pkm_builder with_friendship","url":"gpkm/pkm/pkm_builder.html#with_friendship"},{"doc":"gpkm/pkm/pkm_builder","title":"with_gender","content":"pub fn with_gender(\n  pkm: Pkm,\n  encounter_main_info: List(Int),\n) -&gt; Pkm\n\nSynonyms:\ngpkm/pkm/pkm_builder.with_gender\ngpkm/pkm/pkm_builder with_gender","url":"gpkm/pkm/pkm_builder.html#with_gender"},{"doc":"gpkm/pkm/pkm_builder","title":"with_held_item","content":"pub fn with_held_item(pkm: Pkm, held_item: List(Int)) -&gt; Pkm\n\nSynonyms:\ngpkm/pkm/pkm_builder.with_held_item\ngpkm/pkm/pkm_builder with_held_item","url":"gpkm/pkm/pkm_builder.html#with_held_item"},{"doc":"gpkm/pkm/pkm_builder","title":"with_hidden_power","content":"pub fn with_hidden_power(pkm: Pkm) -&gt; Pkm\n\nSynonyms:\ngpkm/pkm/pkm_builder.with_hidden_power\ngpkm/pkm/pkm_builder with_hidden_power","url":"gpkm/pkm/pkm_builder.html#with_hidden_power"},{"doc":"gpkm/pkm/pkm_builder","title":"with_individual_values","content":"pub fn with_individual_values(\n  pkm: Pkm,\n  individual_values: List(Int),\n) -&gt; Pkm\n\nSynonyms:\ngpkm/pkm/pkm_builder.with_individual_values\ngpkm/pkm/pkm_builder with_individual_values","url":"gpkm/pkm/pkm_builder.html#with_individual_values"},{"doc":"gpkm/pkm/pkm_builder","title":"with_level","content":"pub fn with_level(pkm: Pkm, level: List(Int)) -&gt; Pkm\n\nSynonyms:\ngpkm/pkm/pkm_builder.with_level\ngpkm/pkm/pkm_builder with_level","url":"gpkm/pkm/pkm_builder.html#with_level"},{"doc":"gpkm/pkm/pkm_builder","title":"with_moves","content":"pub fn with_moves(pkm: Pkm, bs: UnencryptedPkmBytes) -&gt; Pkm\n Uses the whole UnencryptedPkmBytes to build Pkm with `moves`\n\n As each move/pp is a field of UnencryptedPkmBytes\n each move/pp will be extracted from the `get_moves` function\n\nSynonyms:\ngpkm/pkm/pkm_builder.with_moves\ngpkm/pkm/pkm_builder with_moves","url":"gpkm/pkm/pkm_builder.html#with_moves"},{"doc":"gpkm/pkm/pkm_builder","title":"with_national_pokedex_id","content":"pub fn with_national_pokedex_id(\n  pkm: Pkm,\n  national_pokedex_id: List(Int),\n) -&gt; Pkm\n\nSynonyms:\ngpkm/pkm/pkm_builder.with_national_pokedex_id\ngpkm/pkm/pkm_builder with_national_pokedex_id","url":"gpkm/pkm/pkm_builder.html#with_national_pokedex_id"},{"doc":"gpkm/pkm/pkm_builder","title":"with_nature","content":"pub fn with_nature(pkm: Pkm, pid: List(Int)) -&gt; Pkm\n\nSynonyms:\ngpkm/pkm/pkm_builder.with_nature\ngpkm/pkm/pkm_builder with_nature","url":"gpkm/pkm/pkm_builder.html#with_nature"},{"doc":"gpkm/pkm/pkm_builder","title":"with_nickname","content":"pub fn with_nickname(pkm: Pkm, nickname: List(Int)) -&gt; Pkm\n\nSynonyms:\ngpkm/pkm/pkm_builder.with_nickname\ngpkm/pkm/pkm_builder with_nickname","url":"gpkm/pkm/pkm_builder.html#with_nickname"},{"doc":"gpkm/pkm/pkm_builder","title":"with_original_language","content":"pub fn with_original_language(\n  pkm: Pkm,\n  original_language: List(Int),\n) -&gt; Pkm\n\nSynonyms:\ngpkm/pkm/pkm_builder.with_original_language\ngpkm/pkm/pkm_builder with_original_language","url":"gpkm/pkm/pkm_builder.html#with_original_language"},{"doc":"gpkm/pkm/pkm_builder","title":"with_ot_id","content":"pub fn with_ot_id(pkm: Pkm, ot_id: List(Int)) -&gt; Pkm\n\nSynonyms:\ngpkm/pkm/pkm_builder.with_ot_id\ngpkm/pkm/pkm_builder with_ot_id","url":"gpkm/pkm/pkm_builder.html#with_ot_id"},{"doc":"gpkm/pkm/pkm_builder","title":"with_ot_name","content":"pub fn with_ot_name(pkm: Pkm, ot_name: List(Int)) -&gt; Pkm\n\nSynonyms:\ngpkm/pkm/pkm_builder.with_ot_name\ngpkm/pkm/pkm_builder with_ot_name","url":"gpkm/pkm/pkm_builder.html#with_ot_name"},{"doc":"gpkm/pkm/pkm_builder","title":"with_ot_secret_id","content":"pub fn with_ot_secret_id(\n  pkm: Pkm,\n  ot_secret_id: List(Int),\n) -&gt; Pkm\n\nSynonyms:\ngpkm/pkm/pkm_builder.with_ot_secret_id\ngpkm/pkm/pkm_builder with_ot_secret_id","url":"gpkm/pkm/pkm_builder.html#with_ot_secret_id"},{"doc":"gpkm/pkm/pkm_builder","title":"with_pid","content":"pub fn with_pid(pkm: Pkm, pid: List(Int)) -&gt; Pkm\n\nSynonyms:\ngpkm/pkm/pkm_builder.with_pid\ngpkm/pkm/pkm_builder with_pid","url":"gpkm/pkm/pkm_builder.html#with_pid"},{"doc":"gpkm/pkm/pkm_builder","title":"with_shiny","content":"pub fn with_shiny(\n  pkm: Pkm,\n  pid: List(Int),\n  ot_id: List(Int),\n  secret_id: List(Int),\n) -&gt; Pkm\n\nSynonyms:\ngpkm/pkm/pkm_builder.with_shiny\ngpkm/pkm/pkm_builder with_shiny","url":"gpkm/pkm/pkm_builder.html#with_shiny"},{"doc":"gpkm/pkm/pkm_builder","title":"with_species","content":"pub fn with_species(\n  pkm: Pkm,\n  national_pokedex_id: List(Int),\n) -&gt; Pkm\n\nSynonyms:\ngpkm/pkm/pkm_builder.with_species\ngpkm/pkm/pkm_builder with_species","url":"gpkm/pkm/pkm_builder.html#with_species"},{"doc":"gpkm/pkm/pkm_builder","title":"gpkm/pkm/pkm_builder","content":"","url":"gpkm/pkm/pkm_builder.html"},{"doc":"gpkm/utils/files","title":"read_file","content":"pub fn read_file(path: String) -&gt; Result(BitArray, a)\n\nSynonyms:\ngpkm/utils/files.read_file\ngpkm/utils/files read_file","url":"gpkm/utils/files.html#read_file"},{"doc":"gpkm/utils/files","title":"read_file_as_str","content":"pub fn read_file_as_str(path: String) -&gt; Result(String, Nil)\n\nSynonyms:\ngpkm/utils/files.read_file_as_str\ngpkm/utils/files read_file_as_str","url":"gpkm/utils/files.html#read_file_as_str"},{"doc":"gpkm/utils/files","title":"write_file","content":"pub fn write_file(\n  path: String,\n  bytes: BitArray,\n) -&gt; Result(Nil, a)\n\nSynonyms:\ngpkm/utils/files.write_file\ngpkm/utils/files write_file","url":"gpkm/utils/files.html#write_file"},{"doc":"gpkm/utils/files","title":"gpkm/utils/files","content":"","url":"gpkm/utils/files.html"},{"doc":"gpkm/pkm/ser_de","title":"Errors","content":"pub type Errors {\n  InvalidSize(dt: EncryptedDataType)\n  ReadError(dt: EncryptedDataType)\n  UnhandledDataType(dt: EncryptedDataType)\n}\n\nInvalidSize(dt: EncryptedDataType)\n\n\nReadError(dt: EncryptedDataType)\n\n\nUnhandledDataType(dt: EncryptedDataType)\n\n\nSynonyms:\ngpkm/pkm/ser_de.Errors\ngpkm/pkm/ser_de Errors","url":"gpkm/pkm/ser_de.html#Errors"},{"doc":"gpkm/pkm/ser_de","title":"as_binary","content":"pub fn as_binary(\n  bits_read: Result(BitArray, Nil),\n  data_type: EncryptedDataType,\n) -&gt; Result(BitArray, Nil)\n Converts BitArray&#39;s encoding to Bin:\n - if the BitArray&#39;s content is already Bin, it remains unchained\n - else, it decodes the B64 BitArray\n\nSynonyms:\ngpkm/pkm/ser_de.as_binary\ngpkm/pkm/ser_de as_binary","url":"gpkm/pkm/ser_de.html#as_binary"},{"doc":"gpkm/pkm/ser_de","title":"build_pkm_from_bytes","content":"pub fn build_pkm_from_bytes(pbs: UnencryptedPkmBytes) -&gt; Pkm\n Construct Pkm record,\n by selecting, then converting pkm_bytes fields\n\n This Pkm record will be serialized to JSON\n\nSynonyms:\ngpkm/pkm/ser_de.build_pkm_from_bytes\ngpkm/pkm/ser_de build_pkm_from_bytes","url":"gpkm/pkm/ser_de.html#build_pkm_from_bytes"},{"doc":"gpkm/pkm/ser_de","title":"deserialize_b64","content":"pub fn deserialize_b64(b64: String) -&gt; Result(BitArray, Nil)\n Deserialize base64 string, no matter its format:\n - single line\n - multiple spaced lines\n\nSynonyms:\ngpkm/pkm/ser_de.deserialize_b64\ngpkm/pkm/ser_de deserialize_b64","url":"gpkm/pkm/ser_de.html#deserialize_b64"},{"doc":"gpkm/pkm/ser_de","title":"get_current_pkm_bytes_size","content":"pub fn get_current_pkm_bytes_size(\n  bits_read: Result(BitArray, Nil),\n) -&gt; Result(Int, Nil)\n\nSynonyms:\ngpkm/pkm/ser_de.get_current_pkm_bytes_size\ngpkm/pkm/ser_de get_current_pkm_bytes_size","url":"gpkm/pkm/ser_de.html#get_current_pkm_bytes_size"},{"doc":"gpkm/pkm/ser_de","title":"get_unencrypted_header","content":"pub fn get_unencrypted_header(\n  pkm_bytes: List(Int),\n  endian: Endian,\n) -&gt; UnencryptedHeaderBytes\n\nSynonyms:\ngpkm/pkm/ser_de.get_unencrypted_header\ngpkm/pkm/ser_de get_unencrypted_header","url":"gpkm/pkm/ser_de.html#get_unencrypted_header"},{"doc":"gpkm/pkm/ser_de","title":"get_unencrypted_pkm_bytes","content":"pub fn get_unencrypted_pkm_bytes(\n  pkm_bits: BitArray,\n  endian: Endian,\n) -&gt; UnencryptedPkmBytes\n\nSynonyms:\ngpkm/pkm/ser_de.get_unencrypted_pkm_bytes\ngpkm/pkm/ser_de get_unencrypted_pkm_bytes","url":"gpkm/pkm/ser_de.html#get_unencrypted_pkm_bytes"},{"doc":"gpkm/pkm/ser_de","title":"header_as_bytes","content":"pub fn header_as_bytes(h: UnencryptedHeaderBytes) -&gt; List(Int)\n\nSynonyms:\ngpkm/pkm/ser_de.header_as_bytes\ngpkm/pkm/ser_de header_as_bytes","url":"gpkm/pkm/ser_de.html#header_as_bytes"},{"doc":"gpkm/pkm/ser_de","title":"read_pkm","content":"pub fn read_pkm(\n  pkm_data: String,\n  dt: EncryptedDataType,\n) -&gt; Result(Pkm, Errors)\n\nSynonyms:\ngpkm/pkm/ser_de.read_pkm\ngpkm/pkm/ser_de read_pkm","url":"gpkm/pkm/ser_de.html#read_pkm"},{"doc":"gpkm/pkm/ser_de","title":"slice_bits","content":"pub fn slice_bits(\n  bits_read: Result(BitArray, Nil),\n  dt: EncryptedDataType,\n) -&gt; Result(BitArray, Errors)\n\nSynonyms:\ngpkm/pkm/ser_de.slice_bits\ngpkm/pkm/ser_de slice_bits","url":"gpkm/pkm/ser_de.html#slice_bits"},{"doc":"gpkm/pkm/ser_de","title":"gpkm/pkm/ser_de","content":"","url":"gpkm/pkm/ser_de.html"},{"doc":"gpkm","title":"main","content":"pub fn main() -&gt; Result(Pkm, Errors)\n\nSynonyms:\ngpkm.main\ngpkm main","url":"gpkm.html#main"},{"doc":"gpkm","title":"gpkm","content":"","url":"gpkm.html"},{"doc":"gpkm/sav/blocks","title":"Action","content":"pub type Action {\n  Encrypt\n  Decrypt\n}\n\nEncrypt\n\n\nDecrypt\n\n\nSynonyms:\ngpkm/sav/blocks.Action\ngpkm/sav/blocks Action","url":"gpkm/sav/blocks.html#Action"},{"doc":"gpkm/sav/blocks","title":"Blocks","content":"pub type Blocks {\n  Blocks(a: Bytes, b: Bytes, c: Bytes, d: Bytes)\n}\n\nBlocks(a: Bytes, b: Bytes, c: Bytes, d: Bytes)\n\n\nSynonyms:\ngpkm/sav/blocks.Blocks\ngpkm/sav/blocks Blocks","url":"gpkm/sav/blocks.html#Blocks"},{"doc":"gpkm/sav/blocks","title":"blocks_as_bytes","content":"pub fn blocks_as_bytes(bs: Blocks) -&gt; List(Int)\n\nSynonyms:\ngpkm/sav/blocks.blocks_as_bytes\ngpkm/sav/blocks blocks_as_bytes","url":"gpkm/sav/blocks.html#blocks_as_bytes"},{"doc":"gpkm/sav/blocks","title":"bytes_as_blocks","content":"pub fn bytes_as_blocks(bs: List(Int)) -&gt; Blocks\n Construct Blocks record from block bytes\n each block is 32 bytes long\n\nSynonyms:\ngpkm/sav/blocks.bytes_as_blocks\ngpkm/sav/blocks bytes_as_blocks","url":"gpkm/sav/blocks.html#bytes_as_blocks"},{"doc":"gpkm/sav/blocks","title":"rand","content":"pub fn rand(seed: Int) -&gt; Int\n\nSynonyms:\ngpkm/sav/blocks.rand\ngpkm/sav/blocks rand","url":"gpkm/sav/blocks.html#rand"},{"doc":"gpkm/sav/blocks","title":"shift_personality_value","content":"pub fn shift_personality_value(pid: List(Int)) -&gt; Int\n Shift personality value (PV, a.k.a: pid)\n accoring to the following formula:\n\n c\n shift = ((PV &amp; 0x3E000) &gt;&gt; 0xD) % 24\n \n\nSynonyms:\ngpkm/sav/blocks.shift_personality_value\ngpkm/sav/blocks shift_personality_value","url":"gpkm/sav/blocks.html#shift_personality_value"},{"doc":"gpkm/sav/blocks","title":"shuffle_block_bytes","content":"pub fn shuffle_block_bytes(\n  bs: List(Int),\n  pid: List(Int),\n  ord: Action,\n) -&gt; List(Int)\n Shuffle blocks according to the specifies rules:\n https://projectpokemon.org/home/docs/gen-4/pkm-structure-r65/\n\nSynonyms:\ngpkm/sav/blocks.shuffle_block_bytes\ngpkm/sav/blocks shuffle_block_bytes","url":"gpkm/sav/blocks.html#shuffle_block_bytes"},{"doc":"gpkm/sav/blocks","title":"gpkm/sav/blocks","content":"","url":"gpkm/sav/blocks.html"},{"doc":"gpkm/sav/decrypt","title":"decrypt_pkm","content":"pub fn decrypt_pkm(pkm_bits: BitArray) -&gt; List(Int)\n\nSynonyms:\ngpkm/sav/decrypt.decrypt_pkm\ngpkm/sav/decrypt decrypt_pkm","url":"gpkm/sav/decrypt.html#decrypt_pkm"},{"doc":"gpkm/sav/decrypt","title":"gpkm/sav/decrypt","content":"","url":"gpkm/sav/decrypt.html"},{"doc":"gpkm/sav/encrypt","title":"gpkm/sav/encrypt","content":"","url":"gpkm/sav/encrypt.html"}]);